#include <PID_v1.h>
#include <Arduino.h>
#include "motorEncoder.h"

MotorEncoder::MotorEncoder(uint8_t encoder0pinA, uint8_t encoder0pinB, int E_left, int M_left, float ratio, float radius) {
    m_encoder0pinA = encoder0pinA; // pin 2 or 3
    m_encoder0pinB = encoder0pinB;
    m_E_left = E_left;// pwm pin !
    m_M_left = M_left;
    m_ratio = ratio;
    m_radius = radius;
    setPoint = 0;
    direction = true;  // Default -> Forward
    pinMode(m_encoder0pinB, INPUT);
    pinMode(m_M_left, OUTPUT);  // L298P Control port settings DC motor driver board for the output mode
    pinMode(m_E_left, OUTPUT);

    // Initialize PID controller
    myPID = new PID(&abs_duration, &val_output, &setPoint, Kp, Ki, Kd, DIRECT);
    myPID->SetMode(AUTOMATIC);  // PID is set to automatic mode
    myPID->SetSampleTime(100);  // Set PID sampling frequency to 100ms
}

void MotorEncoder::wheelSpeed() {
  int Lstate = digitalRead(m_encoder0pinA);
  if((encoder0PinALast == LOW) && Lstate==HIGH){
    int val = digitalRead(m_encoder0pinB);
    if(val == LOW && direction){
      direction = false; //Reverse
    }
    else if(val == HIGH && !direction){
      direction = true;  //Forward
    }
  }
  encoder0PinALast = Lstate;

  if(!direction)  duration++;
  else  duration--;

}

void MotorEncoder::EncoderHandlerInit(void (*ISR)(void)) {
    direction = true;  // Default -> Forward
    pinMode(m_encoder0pinB, INPUT);  // Ensure encoder pin A is set as INPUT
    attachInterrupt(m_encoder0pinA-2, ISR, CHANGE);
}

void MotorEncoder::advance() {
    digitalWrite(m_M_left, LOW);
    analogWrite(m_E_left, abs(val_output));
}

void MotorEncoder::back() {
    digitalWrite(m_M_left, HIGH);
    analogWrite(m_E_left, abs(val_output));
}

void MotorEncoder::stop() {
    analogWrite(m_E_left, 0); 
}

double MotorEncoder::compute(int debug) {
    abs_duration = abs(duration);
    result = myPID->Compute();
    if (result) {
        if (debug) {
            Serial.print("Pluse: ");
            Serial.print(duration);
            Serial.print("  Power: ");
            Serial.println(val_output);
            
        }
        double old_duration = duration;
        duration = 0;
        return old_duration;
    }
    return duration;
}